from dash import Dash, Input, Output, State, ALL
import plotly.io as pio
import dash_bootstrap_components as dbc
import sys
from amplabs.components.appLayout import setup_layout
from amplabs.variables import *
import amplabs.variables as avs
from amplabs.callbacks.toggleNavbar import toggle_navbar_collapse
from amplabs.callbacks.updateLineChart import update_line_chart
import platform

app = Dash(__name__, external_stylesheets=[dbc.themes.BOOTSTRAP])


if platform.system() == 'Darwin':
    pio.renderers.default = 'svg'


class plot:
    instances = []

    def __init__(self, df, df_name=None, x_columns=None, y_columns=None):
        try:
            self.df = df
            self.df_name = df_name
            self.xcolumn = x_columns
            self.ycolumn = y_columns
            self.headers = self.df.columns.tolist()
            plot.instances.append(self)
        except TypeError as dfe:
            print(f"Error: {dfe}")
            sys.exit(1)

    def set_color(self, colors):
        self.colors = colors

    def set_xrange(self, ranges):
        self.xrange = ranges

    def set_yrange(self, ranges):
        self.yrange = ranges

    def set_xintervals(self, intervals):
        self.xinterval = intervals

    def set_yintervals(self, intervals):
        self.yinterval = intervals

    def set_xlabel(self, label):
        self.xlabel = label

    def set_ylabel(self, label):
        self.ylabel = label

    def set_linewidth(self, width):
        try:
            # Show an error when you receive a string here
            if not isinstance(width, (int, float)):
                raise TypeError("Width must be an int or float")
            self.line_width = width
        except TypeError as ve:
            print(f"Error:{ve}")
            sys.exit(1)

    @classmethod
    def reset_all(cls):
        #Clear the instance 
        cls.instances = []
        # Clear global variables
        data_list.clear()
        list_of_headers.clear()
        color_list.clear()
        x_ranges.clear()
        y_ranges.clear()
        x_interval.clear()
        y_interval.clear()
        line_width.clear()
        ylabel.clear()
        x_columns.clear()
        y_columns.clear()        

    @classmethod
    def plot_on_dashboard(cls):
        for instance in cls.instances:
            data_list.append(instance.df)
            data_names.append(instance.df_name)
            list_of_headers.append(instance.headers)
            color_list.append(getattr(instance, "colors", []))
            x_ranges.append(getattr(instance, "xrange", []))
            y_ranges.append(getattr(instance, "yrange", []))
            x_interval.append(getattr(instance, "xinterval", []))
            y_interval.append(getattr(instance, "yinterval", []))
            line_width.append(getattr(instance, "line_width", 2.5))
            ylabel.append(getattr(instance, "ylabel", None))
            avs.xlabel = (
                getattr(instance, "xlabel", None)
                if isinstance(getattr(instance, "xlabel", None), str)
                else avs.xlabel
            )
        try:
            # add callback for toggling the collapse on small screens
            app.callback(
                Output("navbar-collapse", "is_open"),
                [Input("navbar-toggler", "n_clicks")],
                [State("navbar-collapse", "is_open")],
            )(toggle_navbar_collapse)

            app.layout = setup_layout()

            app.callback(
                Output("graph", "figure"),
                [Input({"type": "add-x", "index": ALL}, "value")],
                [Input({"type": "add-y", "index": ALL}, "value")],
            )(update_line_chart)

        except ValueError as ve:
            print(f"Error: {ve}")
            sys.exit(1)
        except Exception as ve:
            print(f"Error: {ve}")
            sys.exit(1)

        start_dash_server()

    @classmethod
    def plot_as_svg(cls):
        try:
            for instance in cls.instances:
                data_list.append(instance.df)
                list_of_headers.append(instance.headers)
                color_list.append(getattr(instance, "colors", []))
                x_ranges.append(getattr(instance, "xrange", []))
                y_ranges.append(getattr(instance, "yrange", []))
                x_interval.append(getattr(instance, "xinterval", []))
                y_interval.append(getattr(instance, "yinterval", []))
                line_width.append(getattr(instance, "line_width", 2))
                ylabel.append(getattr(instance, "ylabel", None))
                avs.xlabel = (
                    getattr(instance, "xlabel", None)
                    if isinstance(getattr(instance, "xlabel", None), str)
                    else avs.xlabel
                )
                x_columns.append(instance.xcolumn)
                y_columns.append(instance.ycolumn)
            fig = update_line_chart(x_columns, y_columns)
            fig.show()
            cls.reset_all()
        except ValueError as ve:
            print(f"Error: {ve}")
            sys.exit(1)
        except Exception as ve:
            print(f"Error: {ve}")
            sys.exit(1)


def start_dash_server():
    global dash_server_running
    app.run_server(debug=True)



from dash import dcc
import dash_bootstrap_components as dbc


def generateDropDowns(column_headers, index):
    dropdownLayout = dbc.Row(
        [
            dbc.Col(dbc.Label("X-Axis:"), width="auto"),
            dbc.Col(
                dcc.Dropdown(
                    id={"type": "add-x", "index": index},
                    options=column_headers[index],
                    value="",
                    placeholder="Select x axis",
                    style={
                        "width": "100%",
                        "display": "inline-block",
                        "alignSelf": "center",
                    },
                ),
            ),
            dbc.Col(dbc.Label("Y-Axis:"), width="auto"),
            dbc.Col(
                dcc.Dropdown(
                    id={"type": "add-y", "index": index},
                    options=column_headers[index],
                    value=[],
                    multi=True,
                    placeholder="Select y axis",
                    style={
                        "width": "100%",
                        "display": "inline-block",
                        "alignSelf": "center",
                    },
                ),
            ),
        ],
        style={
            "marginTop": "2%",
            "marginLeft": "5%",
            "alignItems": "center",
        },
    )
    return dropdownLayout


def generate_axes_selection(file_names, column_headers):
    items = []

    for index, file_path in enumerate(file_names):
        items.append(dbc.ListGroupItem(file_path.lstrip("./")))
        items.append(generateDropDowns(column_headers, index))
    return items


def htmlSelectionBar(list_of_file_names, list_of_headers):
    selectionBar = dbc.Card(
        dbc.CardBody(
            [
                dbc.ListGroup(
                    generate_axes_selection(list_of_file_names, list_of_headers),
                    numbered=True,
                    id="axes_selection",
                )
            ],
            className="w-100",
        ),
        className="mb-3 d-flex align-items-center w-100",
    )
    return selectionBar



from dash import html
import dash_bootstrap_components as dbc
from PIL import Image

import os

# Get the directory where this script is located
script_dir = os.path.dirname(os.path.abspath(__file__))

# Construct the absolute path to the "assets" directory at the root of the project
project_root = os.path.abspath(os.path.join(script_dir, "../../"))
assets_dir = os.path.join(project_root, "assets")

# Construct the absolute path to the "favicon.ico" file
image_path = os.path.join(assets_dir, "favicon.png")

AMPLABS_LOGO = Image.open(image_path)


HTML_NAVBAR = dbc.Navbar(
    [
        html.A(
            # Use row and col to control vertical alignment of logo / brand
            dbc.Row(
                [
                    dbc.Col(html.Img(src=AMPLABS_LOGO, height="30px")),
                    dbc.Col(dbc.NavbarBrand("AmpLabs", className="ms-2")),
                ],
                align="center",
                className="g-0",
            ),
            href="https://plotly.com",
            style={"textDecoration": "none", "marginLeft": "1%"},
        ),
        dbc.NavbarToggler(id="navbar-toggler", n_clicks=0),
        dbc.Collapse(
            dbc.Row(
                [
                    dbc.Col(
                        dbc.NavItem(
                            dbc.NavLink(
                                "About Us",
                                href="https://www.amplabs.ai/",
                                target="_blank",
                            )
                        ),
                    ),
                ],
                style={"color": "white"},
            ),
            id="navbar-collapse",
            is_open=False,
            navbar=True,
            style={
                "justifyContent": "end",
                "marginRight": "5%",
            },
        ),
    ],
    color="dark",
    dark=True,
)



from setuptools import find_packages, setup

with open('README.md', 'r', encoding='utf-8') as file:
    long_description = file.read()

setup(
    name='amplabs',
    packages=find_packages(),
    package_data={'assets': ['*']},
    version='0.1.9',
    description='One of the AmpLabs product to create high end plots for your data',
    author='Amplabs',
    install_requires=['dash','Pillow','dash-bootstrap-components'],
    readme = "README.md",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True
)